﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Silverlight;
using Infragistics.Silverlight.Controls;

namespace SilverlightApplication35
{
    public partial class MainPage : UserControl
    {
        const int MOVES_TO_AUTO_EXPAND = 3;

        private int _mouseMoves = 0;

        public MainPage()
        {
            InitializeComponent();
        }

        private void XamWebTree_ItemDragMove(object sender, DragMoveEventArgs e)
        {
            const int offset = 10;
            Rect treeBounds;
            Rect dragScrollArea;
            XamWebTree tree = sender as XamWebTree;
            List<XamWebTreeItem> items = new List<XamWebTreeItem>();
            bool isDragUp = true;
            XamWebTreeItem treeItem = null;
            XamWebTreeItem movingOverTreeItem = null;

            movingOverTreeItem = this.GetTreeItemMouseOver(e.CursorPosition, tree);
            if ((movingOverTreeItem != null) &&
                (movingOverTreeItem.HasChildren == true) && (movingOverTreeItem.IsExpanded == false))
                    movingOverTreeItem.IsExpanded = true;

            // determine the bounds of the area in which the tree is visualized
            treeBounds = this.GetBounds(tree);

            // define an area for which causes scroll up while dragging
            dragScrollArea = treeBounds;
            dragScrollArea.Height = offset;

            // check if the mouse position is in that area, 
            // if not define area for scrolling down while dragging
            if (!dragScrollArea.Contains(e.CursorPosition))
            {
                isDragUp = false;

                dragScrollArea = treeBounds;
                dragScrollArea.Y = treeBounds.Y + treeBounds.Height - offset;
                dragScrollArea.Height = offset;
            }

            // check if the mouse position is in the scroll area set above, 
            // if yes, find the outermost tree item and invoke ScrollIntoView
            if (dragScrollArea.Contains(e.CursorPosition))
            {
                // find all UI elements in the scroll area and filter tree items only 
                items.AddRange(
                    VisualTreeHelper.FindElementsInHostCoordinates(dragScrollArea, tree)
                    .Where(o => o is XamWebTreeItem)
                    .Cast<XamWebTreeItem>()
                    );

                // if there is at least one tree item ...
                try
                {
                    treeItem = items.First();
                }
                catch (InvalidOperationException ex)
                {
                    return;
                }

                // ... perform finding the outermost tree item by its Y coordinate
                if (isDragUp)
                    items.ForEach((el) => { if (this.GetBounds(treeItem).Y > this.GetBounds(el).Y) treeItem = el; });
                else
                    items.ForEach((el) => { if (this.GetBounds(treeItem).Y < this.GetBounds(el).Y) treeItem = el; });

                // NOTE: the next item to be scrolled seems to be always found with the algorithm above,
                // so just scroll it into view
                treeItem.ScrollItemIntoView();
            }
        }

        private XamWebTreeItem GetTreeItemMouseOver(Point point, XamWebTree tree)
        {
            XamWebTreeItem item = null;

            List<UIElement> elements = new List<UIElement>(VisualTreeHelper.FindElementsInHostCoordinates(point, tree));

            try
            {
                item = elements.First(e => e is XamWebTreeItem) as XamWebTreeItem;
            }
            catch (Exception)
            {
                item = null;
            }

            return item;
        }

        public Rect GetBounds(FrameworkElement element)
        {
            GeneralTransform transform;
            Point offset;
            Point transformedOffset;

            transform = element.TransformToVisual(Application.Current.RootVisual);
            offset = transform.Transform(new Point(0, 0));

            transformedOffset = transform.Transform(new Point(element.ActualWidth, element.ActualHeight));
            var transformedWidth = transformedOffset.X - offset.X;
            var transformedHeight = transformedOffset.Y - offset.Y;

            return new Rect(offset.X, offset.Y, transformedWidth, transformedHeight);
        }
    }
}
